# parameter_assessment.py plot the parameter of a set or compares sets of paramters
import sys,os
import numpy as np
import matplotlib.pyplot as plt
import numpy.ma as ma


######## plots the most negative and most positive parameter in the sets #######
# for a lennard jones potential it plots the minimum energie and the LJ-radi

if '--plotparams' in sys.argv:
  parname = sys.argv[sys.argv.index('--plotparams')+1]
  parameter = np.genfromtxt(parname,skip_header=1)
  atyps = len(parameter[0])
  if '--attractparameter' in sys.argv:
    power1 = float(sys.argv[sys.argv.index('--attractparameter')+1])
    power2 = float(sys.argv[sys.argv.index('--attractparameter')+2])
    Emin = ((power2/power1)**(power1/(power1-power2))-(power2/power1)**(power2/(power1-power2)))*parameter[atyps:2*atyps]*parameter[2*atyps:]
    rmin = ((power1/power2)**(1.0/(power1-power2)))*parameter[:atyps]
    parameter = np.append(rmin,Emin, axis = 0)
  
  if '--namelist' in sys.argv:
    names = np.genfromtxt(sys.argv[sys.argv.index('--namelist')+1], dtype = str)[:,-1]
  else:
    names = np.arange(1,len(parameter)+1,1).astype(str)
  
  if len(np.shape(parameter))>1:
    bins = len(parameter)/atyps
    sortpar = np.zeros((bins, (atyps*(atyps-1)/2)+atyps))
    sortnames = []
    ind = 0

    for n in range(atyps):
      for nn in range(n,atyps):
	for b in range(bins):
	  sortpar[b,ind] = parameter[b*atyps+n,nn]
	sortnames.append(names[n]+' / '+names[nn])
	ind += 1
    sortnames = np.array(sortnames, dtype = str)
    for b in range(bins):
      print 'bins ', b
      sort = np.argsort(sortpar[b])
      sortpar[b] = sortpar[b][sort]
      sortname = sortnames[sort]
      print ' unfavorable contacts               |                  favorable contacts '
      for i in range(30):
	print "%32s" % sortname[-i-1], "%3.3f" % sortpar[b][-i-1],"%32s" % sortname[i], "%3.3f" % sortpar[b][i]
      
      if '--sortplot' in sys.argv:
	fig=plt.figure('best')
	ax=fig.add_subplot(111)
	ax.set_ylabel('$\mathrm{p}$',rotation=0,fontsize=15)
	ax.xaxis.set_ticks_position('top')
	ax.set_xticks(np.arange(0,30))
	ax.set_xticklabels(sortname[:30],rotation=30 , ha='left')
	p=ax.bar(np.arange(0,30),sortpar[b][:30],0.5)
	fig1=plt.figure('worst')
	ax1=fig1.add_subplot(111)
	ax1.set_ylabel('$\mathrm{p}$',rotation=0, fontsize=15)
	ax1.set_xticks(np.arange(0,30))
	ax1.set_xticklabels(sortname[-30:], rotation =-30,ha='left')
	p1=ax1.bar(np.arange(0,30),sortpar[b][-30:][-np.arange(1,31)],0.5)
	plt.show()
	
	
	
    xnames = names
    ynames = []
    for i in range(bins):  
      fig=plt.figure(parname)
      ax=fig.add_subplot(111)
      p=ax.imshow(parameter[i*atyps:(i+1)*atyps],cmap='spectral',interpolation='none')
      ax.set_yticks(np.arange(0, atyps, 1))
      ax.set_xticks(np.arange(0, atyps, 1))
      ax.set_xticklabels(xnames, rotation = 90)
      ax.set_yticklabels(xnames)
      fig.colorbar(p)
      plt.show()
  
  else:
    parameter = parameter.reshape((1,len(parameter)))
    xnames = names
    fig=plt.figure(parname)
    ax=fig.add_subplot(111)
    p=ax.imshow(parameter,cmap='spectral',interpolation='none')
    ax.set_xticks(np.arange(0, len(parameter[0]), 1))
    ax.set_xticklabels(xnames, rotation = 90)
    fig.colorbar(p)
    plt.show()



##### plots a matrix with the correlation and the error between different different sets of parameters of the same type


if '--crossparameter' in sys.argv:
  numcross = int(sys.argv[sys.argv.index('--crossparameter')+1])
  parameter = []
  aton = False
  
  if '--attractparameter' in sys.argv:
    aton = True
    power1 = float(sys.argv[sys.argv.index('--attractparameter')+1])
    power2 = float(sys.argv[sys.argv.index('--attractparameter')+2])
  
  erron = False
  if '--meanerror' in sys.argv:
    erron = True
  
  for i in range(numcross):
    pars = np.genfromtxt(sys.argv[sys.argv.index('--crossparameter')+2+i])
    atyps=len(pars[0])
    if aton:
      Emin = ((power2/power1)**(power1/(power1-power2))-(power2/power1)**(power2/(power1-power2)))*pars[atyps:2*atyps]*pars[2*atyps:]
      rmin = ((power1/power2)**(1.0/(power1-power2)))*pars[:atyps]
      sign = pars[2*atyps:]
      pars = np.append(rmin,np.append(Emin,sign,axis=0), axis = 0)
    parameter.append(pars)
  
  
  parameter = np.array(parameter)
  
  shapepar = len(np.shape(parameter[0]))
  if shapepar > 1:
    bins=int(np.shape(parameter[0])[0]/atyps)      
    mean=np.zeros((numcross,bins))
    std=np.zeros((numcross,bins))
    for i in range(bins):
      for j in range(numcross):
	std[j,i]=np.std(parameter[j,i*atyps:(i+1)*atyps])
	if aton == False:
	  mean[j,i]=np.mean(parameter[j,i*atyps:(i+1)*atyps])
    
    if aton:
      std[:,2]= np.ones(numcross)
    
    corr=np.zeros(np.shape(parameter[0]))
    err = np.zeros(np.shape(parameter[0]))
    for b in range(bins):
      for n in range(atyps):
	for nn in range(atyps):
	  for i in range(numcross):
	    for j in range(i, numcross): 
	      corr[b*atyps+n,nn]+=(parameter[i,b*atyps+n,nn]-mean[i,b])*(parameter[j,b*atyps+n,nn]-mean[j,b])/(std[j,b]*std[i,b])
	      err[b*atyps+n,nn]+=((parameter[i,b*atyps+n,nn]-mean[i,b])-(parameter[j,b*atyps+n,nn]-mean[j,b]))**2/(std[j,b]*std[i,b])
      corr = corr/float(numcross)
      fig=plt.figure('Correlation')
      ax=fig.add_subplot(111)
      p=ax.imshow(corr[b*atyps:(b+1)*atyps],cmap='spectral',interpolation='none')
      fig.colorbar(p)
      
      if erron:
	err = err/float(numcross)
	fig1=plt.figure('Error')
	ax1=fig1.add_subplot(111)
	p1=ax1.imshow(err[b*atyps:(b+1)*atyps],cmap='spectral',interpolation='none')
	fig1.colorbar(p1)
      
      plt.show() 
   
  else:
    atyps=int(np.shape(parameter[0])[-1])    
    mean=np.zeros(numcross)
    std=np.zeros(numcross)
    for j in range(numcross):
	std[j]=np.std(parameter[j])
	mean[j]=np.mean(parameter[j])
    
    corr=np.zeros((1,len(parameter[0])))
    for n in range(atyps):
      for i in range(numcross):
	for j in range(i, numcross): 
	  corr[0,n]+=(parameter[i,n]-mean[i])*(parameter[j,n]-mean[j])/(std[j]*std[i])
    
    corr = corr/float(numcross)
    fig=plt.figure('Correlation')
    ax=fig.add_subplot(111)
    p=ax.imshow(corr,cmap='spectral',interpolation='none')
    fig.colorbar(p)
    plt.show()


######## plots a difference, and a correlation matrix between two sets of parameters    

if '--parameterdifference' in sys.argv:
  parameter1=np.genfromtxt(sys.argv[sys.argv.index('--parameterdifference')+1],skip_header=1)
  parameter2=np.genfromtxt(sys.argv[sys.argv.index('--parameterdifference')+2],skip_header=1)
  atyps=int(np.shape(parameter1)[-1])
  aton = False
  if '--attractparameter' in sys.argv:
    aton = True
    power1 = float(sys.argv[sys.argv.index('--attractparameter')+1])
    power2 = float(sys.argv[sys.argv.index('--attractparameter')+2])
    
    Emin = ((power2/power1)**(power1/(power1-power2))-(power2/power1)**(power2/(power1-power2)))*parameter1[atyps:2*atyps]*parameter1[2*atyps:]
    rmin = ((power1/power2)**(1.0/(power1-power2)))*parameter1[:atyps]
    sign = parameter1[2*atyps:]
    parameter1 = np.append(rmin,np.append(Emin,sign,axis=0), axis = 0)
  
    Emin = ((power2/power1)**(power1/(power1-power2))-(power2/power1)**(power2/(power1-power2)))*parameter2[atyps:2*atyps]*parameter2[2*atyps:]
    rmin = ((power1/power2)**(1.0/(power1-power2)))*parameter2[:atyps]
    sign = parameter2[2*atyps:]
    parameter2 = np.append(rmin,np.append(Emin,sign,axis=0), axis = 0)
  
  if np.shape(parameter1)==np.shape(parameter2):
    bins=int(np.shape(parameter1)[0]/atyps)
    shapepar = len(np.shape(parameter1))
    
    if aton:
      bins = 2
    
    if shapepar>1:
      mean=np.zeros((2,bins))
      std=np.zeros((2,bins))
      for i in range(bins):
	print 'for bin: ', i
	std[0,i]=np.std(parameter1[i*atyps:(i+1)*atyps])
	std[1,i]=np.std(parameter2[i*atyps:(i+1)*atyps])
	mean[0,i]=np.mean(parameter1[i*atyps:(i+1)*atyps])
	mean[1,i]=np.mean(parameter2[i*atyps:(i+1)*atyps])
	print 'max1', np.amax(parameter1[i*atyps:(i+1)*atyps])
	print 'min1', np.amin(parameter1[i*atyps:(i+1)*atyps])
	print 'max2', np.amax(parameter2[i*atyps:(i+1)*atyps])
	print 'min2', np.amin(parameter2[i*atyps:(i+1)*atyps])
	print 'mean1 ',mean[0,i]
	print 'mean2 ',mean[1,i]
	print 'std1 ',std[0,i]
	print 'std2 ',std[1,i]
	parameter1[i*atyps:(i+1)*atyps]=parameter1[i*atyps:(i+1)*atyps]/std[0,i]
	parameter2[i*atyps:(i+1)*atyps]=parameter2[i*atyps:(i+1)*atyps]/std[1,i]
    else:
      parameter1 = parameter1.reshape((1,atyps))
      parameter2 = parameter2.reshape((1,atyps))
      std1=np.std(parameter1)
      std2=np.std(parameter2)
      mean1=np.mean(parameter1)
      mean2=np.mean(parameter2)
      parameter1=parameter1/std1
      parameter2=parameter2/std2
      
    
    out=np.absolute(parameter1-parameter2)
    out2=np.ones(np.shape(out))-(np.absolute(parameter1-parameter2))/(np.absolute(parameter1)+np.absolute(parameter2))
    out3=parameter1*parameter2
    
    if shapepar > 1:
      for i in range(bins):
	parameter1[:(i+1)*atyps]=parameter1[:(i+1)*atyps]-mean[0,i]/std[0,i]
	parameter2[:(i+1)*atyps]=parameter2[:(i+1)*atyps]-mean[1,i]/std[1,i]
    else:
      parameter1=parameter1-mean1/std1
      parameter2=parameter2-mean2/std2
    
    out5=parameter1*parameter2
    corr=np.mean(out5)
    print 'corr', corr
    rmsd=np.mean(out**2)
    print 'rmsd ', rmsd
    
    fig=plt.figure('scaled difference')
    ax=fig.add_subplot(111)
    p=ax.imshow(out,cmap='spectral',interpolation='none')
    fig.colorbar(p)
    fig2=plt.figure('scaled 1-difference')
    ax2=fig2.add_subplot(111)
    p2=ax2.imshow(out2,cmap='spectral',interpolation='none')
    fig2.colorbar(p2)
    fig3=plt.figure('scaled product')
    ax3=fig3.add_subplot(111)
    p3=ax3.imshow(out3,cmap='spectral',interpolation='none')
    fig3.colorbar(p3)
    fig5=plt.figure('mean shifted product, corr')
    ax5=fig5.add_subplot(111)
    p5=ax5.imshow(out5,cmap='spectral',interpolation='none')
    fig5.colorbar(p5)
    plt.show()
  else:
    print 'parameter must have same shape!'
    sys.exit()
    


  
