#@-Scoring: example.sh

#######!!!!! This is not an executeable file !!!!!!########
# This file is only meant to illustrate the work-flow of the @-scoring tool kit
# Further information can be taken from the attached manual

######## Work-flow ######
# 1. Set up Benchmark with pdb's from unbound and bound constituents of the complexes
# for instance: Vreven, Thom, et al. "Updates to the Integrated Protein–Protein Interaction Benchmarks: Docking Benchmark Version 5 and Affinity Benchmark Version 2." Journal of molecular biology 427.19 (2015): 3031-3041.
# 2. Sampling and evaluation of decoy's quality by Attract or another Docking program for all complexes in the Benchmark!!
# for example http://www.attract.ph.tum.de/services/ATTRACT/attract.html
# Attract_benchmark.tar.gz contains 166 complexes sampled with Attract's unbound rigid-body docking
# 3. Divide Benchmark in training and test set and set up a file containing all complexes in the training set.
# for example: example/Benchtrain.txt

#List of complexes or folders respectively
Benchlist=$(ls -d */)

echo ********************************************
echo ** Divide training set for crossvalidation *
echo ********************************************

# for 5-fold crossvalidation create 5 sets of complexes Benchtrain-cv1...5.txt
python ~/@_scoring/Other/cross-distribution.py --complexes Benchtrain.txt --numtestset 5 --output Benchtrain.txt

echo ********************************************
echo ********** Assigning Atom Types ************
echo ********************************************

#./Execute_bench.sh reduce.sh (alternative Benchmark operation)

for var in $Benchlist
do
cd $var
echo $var
# OPLS representation
python $ATTRACTDIR/../allatom/aareduce.py receptor.pdb receptor-aa.pdb --chain A --dumppatch --pdb2pqr > receptor.mapping
# Attract's coare grained representation
python $ATTRACTTOOLS/reduce.py receptor-aa.pdb receptorr.pdb --chain A > /dev/null
# Gaa representation
python ~/@_scoring/Parametrization/redatom.py --pdb receptor-aa.pdb --atomtypefile atomtypes_gaa.dat --addNH --output receptor-gaa.pdb
# Tobi representation
python ~/@_scoring/Parametrization/redatom.py --pdb receptor-aa.pdb --atomtypefile atomtypes_tobi.dat --addNH --output receptor-tobi.pdb
cd ..
done

echo ********************************************
echo ********** Calculating pre-grids ***********
echo ********************************************

# if Haddock was used!!!!
for var in $Benchlist
do
cd $var
python ~/attract/tools/haddock2attract.py /decoys receptor.had ligand.had
cd ..
done

# if any other program for flexible docking was used!!! (--> see manual)
for var in $Benchlist
do
cd $var
./~/@_scorig/Other/pdb_split.sh decoys.pdb
cd ..
done

# if Attract, Attract-ensemble, Attractpep, Attract-normalmodes or iAttract was used for the creation of decoys or others were translated by haddock2attract or pdb_split.sh!!!!!!

# ./Execute_bench.sh pregrid.sh 

for var in $Benchlist
do
cd $var
#creation of step potential pre-grids 
python ~/@_scoring/Parametrization/collect-function.py ../../out_unbound-sorted-dr.dat --proteinmodel gaa receptor-gaa.pdb ligand-gaa.pdb --gridtype none --polationparams 10. 0. 10. --output pregrid-gaa-bins-10.npy

#creation of vdw pre-grids
python ~/@_scoring/Parametrization/collect-function.py out_unbound-sorted-dr.dat --proteinmodel gaa receptor-gaa.pdb ligand-gaa.pdb --gridtype distances --polationparams 8. 2. 10. --powers 2 8. 6. --output pregrid-gaa-distances-2to10A-power8and6.npy 

#creation of vdw pre-grids
python ~@_scoring/Parametrization/asa.py *A-unboundr.pdb *B-unboundr.pdb out_unbound-sorted-dr.dat atomistic attract --coarse_grained 

#resort pre-grids by capri-stars
python ~/@_scoring/Parametrization/decoysetmixer.py --grids pregrid-gaa-distances-2to10A-power8and6.npy --sorton out_unbound-sorted-dr.capstars 1 --numset 3000 --sortorig

#supplement a grid with near-native structures from another decoy-set
python ~/@_scoring/Parametrization/decoysetmixer.py --supplement --grids pregrid-gaa-bin-7.npy pregrid-gaa-true-interface-bin-7.npy --sorton out_unbound-sorted-dr.capstars 1 out_unbound-true-interface-sorted-dr.capstars --extras 3 out_unbound-sorted-dr.fnat out_unbound-true-interface-sorted-dr.fnat out_unbound-sorted-dr.irmsd out_unbound-true-interface-sorted-dr.irmsd out_unbound-sorted-dr.lrmsd out_unbound-true-interface-sorted-dr.lrmsd --numset 5000

cd ..
done

echo ********************************************
echo *********** Training Parameters ************
echo ********************************************

# Execute training.sh for five crossvalidation-sets alternatively 

#Vdw Potential with constraints
python ~/@_scoring/Parametrization/training-MC.py --complexes Benchtrain-cv1.txt 5 --grid pregrid-gaa-distances-2to10A-power8and6-resorted-capstars.npy 3000 distances 1 2 1 2 --mcparams normal 0.05 300000 positionlinear exponential 100 keepsign saddle 8 6 --evaluate out_unbound-sorted-dr-resorted-capstars.capstars capstars --preparameter random 2 378 --output outname matrix --searchrange 2. 5. -20. 20 -2 2

#step potential training until validation set converges
python ~/@_scoring/Parametrization/training-MC.py --complexes Benchtrain-cv2.txt 5 --grid pregrid-gaa-bin-10-resorted-capstars.npy 3000 step --mcparams adaptive 1. 100000 positionquadratic ziczac 100 normal normal --evaluate out_unbound-sorted-dr-resorted-capstars.capstars capstars --preparameter random 1 378 --converge 1000 --output outname matrix

# spline interpolation for complex potential form used by Attract
python ~/@_scoring/Parametrization/training-MC.py --complexes Benchtrain-cv4.txt 5 --grid pregrid-attract-spline8-2to10-in0_4-resorted-capstars.npy 2000 interpolate 2. 10. 0.4 --mcparams normal 0.05 1000000 positionquadratic ziczac 20 interpolate 0 8 6 --evaluate out_unbound-sorted-dr-resorted-capstars.capstars capstars --preparameter random 3 528 --output outname matrix --searchrange 2. 5. -20. 20 -2 2

# BSA-potential with ordinary least squares fit
python ~/@_scoring/Parametrization/training-glm.py --complexes Benchtrain-cv3.txt 5 --grid surfacegrid-gaa-resorted-capstars.npy 3000 step --functionshape 0 27 --evaluate out_unbound-sorted-dr-resorted-capstars.capstars capstars --regressiontype ols --preprocessing meancomplex --output outname normal



echo ********************************************
echo ***** Taking Average of Parameters *********
echo ********************************************

python average-params.py --scores n_scores MC-Params-gaa-step-cv1.par ... MC-Params-gaa-step-cv5.par --scaleparams --bins 1 --output MC-Params-gaa-step-scaled-average.par

echo ********************************************
echo ********* Rescoring Decoy sets *************
echo ********************************************

#./Execute_bench.sh rescore.sh

for var in $Benchlist
do
cd $var
echo $var

#Free scoring with @rank.py
python ~/@_scoring/Rescoring/@rank.py --input out_unbound-sorted-dr.dat --proteinmodel attract *A-unboundr.pdb *B-unboundr.pdb --vdwpotential ../MC-Paramatrix_attractsaddel8-6-2to10A-resorted_parm.par --shift 2 --vdwcutoff 10 --vdwfunctiontype attract --printout > MC-Paramatrix_attractsaddel8-6-2to10A-resorted_parm.rescore

#rescoring on the pre-calculated grids
python ~/@_scoring/Rescoring/grid-rescore.py pregrid-gaa-bin-10.npy ../MC-Paramatrix_gaa-bin10-scaled-average.par --gridtype step 1 27 > MC-Paramatrix_gaa-bin10-scaled-average.rescore

python ~/@_scoring/Rescoring/grid-rescore.py surfacegrid-gaa.npy ../LinReg-Parameter_gaa-bsa-scaled-average.par --gridtype step 0 27 > LinReg-Parameter_gaa-bsa-scaled-average.rescore

python ~/@_scoring/Rescoring/grid-rescore.py pregrid-gaa-distances-2to10A-power8and6.npy ../MC-Paramatrix_gaadistancespower8-6-to10A.par --gridtype distances 2 27 1 2 1 2 > MC-Paramatrix_gaadistancespower8-6-to10A.rescore

python ~/@_scoring/Rescoring/grid-rescore.py pregrid-attract-spline8-2to10-in0_4.npy ../MC-Paramatrix-attractsaddel8-6-2to10A.par --gridtype interpolate 3 32 2 10 0.4 0 8 6 > MC-Paramatrix_attractsaddel8-6-2to10A.rescore

cd ..
done


echo ********************************************
echo ************** Assessment ******************
echo ********************************************

# Assessment of scoring performance
python @_score/Assessment/scoring_assessment.py --scores 2 out_unbound-sorted-dr.dat MC-Params-gaa-step-scaled-average.rescore --evaluate out_unbound-sorted-dr.capstars 1 --benchmark ../../Benchtrain.txt 1 --names Attract MC_gaa --deletezeros --averages --atleast --barchart --double% --ROCcurve --ROCstructures --plotstructures

# Assessment of parameter set from step and BSA-potentials
python @_score/Assessment/contacts-evaluation.py --grid pregrid-gaa-bin-10.npy --benchmark benchmark --deletezeros --evaluate out_unbound-sorted-dr.capstars 1 --normcontacts --namelist atom-types.dat --plotcontact --plotcontactdist --plotparametercontribution MC-Paramatrix_gaa-step-10-scaled-average.par 20 --normparameter

# Correlation analysis of different scoring functions
python @_score/Assessment/correlation_assessment.py --scores 2 out_unbound-sorted-dr.dat MC-Params-gaa-step-scaled-average.rescore --names Attract MC_gaa --benchmark Benchtrain.txt 1 --classification classes.txt difficulty --evaluate out_unbound-sorted-dr.capstars --Rankerror --Rankcorr --topcorr 0.1 all --symdifference --union --complement 

echo ********************************************
echo *********** Combine Scores *****************
echo ********************************************

# create 5 parameter sets for different training sets for crossvalidation
python @_scoring/Parametrization/combine_score.py --complexes Benchtrain-cv1.txt 5 --scores 3 MC-Paramatrix_gaa-bin10-scaled-average.rescore LinReg-Parameter_gaa-bsa-scaled-average.rescore MC-Paramatrix_gaadistancespower8-6-to10A.rescore --yvalue out_unbound-sorted-dr.capstars capstars --method mc --fitparams positionlinear ziczac 50 normal 0.05 10000 --meanscale --numstruc 5000 --output Combine-parmeter-cv1.par

echo ********************************************
echo *********** Average Parameter **************
echo ********************************************

python average-params.py --scores 5 Combine-parameter-cv1.par ... Combine-parameter-cv5.par --bins 0 --output Combine-parameter-averaged.par

echo ********************************************
echo *********** Rescore Combination ************
echo ********************************************

for var in $Benchlist
do
cd $var

python @_scoring/Parametrization/@rank.py --input out_unbound-sorted-dr.dat --proteinmodel gaa receptor-gaa.pdb ligand-gaa.pdb --vdwpotential MC-Paramatrix_gaadistancespower8-6-to10A.par --vdwcutoff 10. --vdwshift 2. --vdwfunctiontype attract --steppotential MC-Paramatrix_gaa-bin10-scaled-average.par --bins 1 10 --solvation MC-Paramatrix_gaa-bsa-scaled-average.par --watersize 1.4 --combination linear Combine-parameter-averaged.par --printout > Combine-parameter-averaged.rescore 

python @_scoring/Parametrization/combine_rescore.py mclinear ../Combine_parameter-averaged.par --scores 3 MC-Paramatrix_gaa-bin10-scaled-average.rescore LinReg-Parameter_gaa-bsa-scaled-average.rescore MC-Paramatrix_gaadistancespower8-6-to10A.rescore --output Combine-parameter-averaged.rescore

cd ..
done
